@@ -1,6 +1,6 @@
 /******************************************************************************
 
-Project:           Portable command line ISP for NXP LPC1000 / LPC2000 family
+Project:           Portable command line ISP for NXP LPC family
                    and Analog Devices ADUC70xx
 
 Filename:          lpc21isp.c
@@ -10,7 +10,7 @@ Compiler:          Microsoft VC 6/7, Microsoft VS2008, Microsoft VS2010,
 
 Author:            Martin Maurer (Martin.Maurer@clibb.de)
 
-Copyright:         (c) Martin Maurer 2003-2011, All rights reserved
+Copyright:         (c) Martin Maurer 2003-2013, All rights reserved
 Portions Copyright (c) by Aeolus Development 2004 http://www.aeolusdevelopment.com
 
     This file is part of lpc21isp.
@@ -217,7 +217,7 @@ Change-History:
                   Made compileable again for linux and windows
                   Fixed bug in ClearSerialPortBuffers (linux)
 1.53   2008-02-25 Changes by Michael Roth
-                  Get priority of debug messages wih -control right
+                  Get priority of debug messages with -control right
 1.54   2008-03-03 Martin Maurer
                   Try to bring lpc21isp back to a useable state in Windows, Cygwin, Linux and Mac OS.
                   Merged in changes by Erika Stefanini, which were done only for old version 1.49:
@@ -237,7 +237,7 @@ Change-History:
                   software compiled with Borland C++ Builder 5. I had to do some minor changes
                   for Borland (see defined __BORLANDC__) and modified to code slightly to have
                   some simple callbacks for screen i/o (see define INTEGRATED_IN_WIN_APP).
-                  Please notet that I don *not* check / modify the part for AnalogDevices !!
+                  Please note that I do *not* check / modify the part for AnalogDevices !!
                   Besides that I fixed some minor issues:
                   added dcb.fOutxCtsFlow = FALSE and dcb.fOutxDsrFlow = FALSE (sometimes required)
                   Now comparing one character less of answer to "Now launching ... code" command
@@ -293,7 +293,7 @@ Change-History:
                   Added support for multiple HEX files, besed on internal version 1.37B.
                   NOTE: this feature is used in production in 1.37B but is not tested in this version.
                   Added numeric debug level command line switch -debugn, n=[0-5]
-                  Added command line scitch -try n to specify nQuestionMarks limit. Defaul: 100
+                  Added command line scitch -try n to specify nQuestionMarks limit. Default: 100
                   Merged in DoNotStart patch from cgommel_new
                   Static functions declarations moved from lpc21isp.h to this file
                   Modified LoadFile() to return error_code instead exit(1)
@@ -340,13 +340,61 @@ Change-History:
                   Added chip id's for more LPC11xx parts, per UM10398 Rev. 11 26 July 2012
 1.85   2012-12-13 Philip Munts
                   Fixed conditional compilation logic in lpc21isp.h to allow compiling for ARM Linux.
+1.86   2012-12-14 diskrepairman
+                  Added devices: LPC1114/203 LPC1114/303 LPC1114/323 LPC1114/333 LPC1115/303
+1.87   2012-12-18 Philip Munts
+                  Added a section of code to ResetTarget() in lpc21isp.c to allow using Linux GPIO pin
+                  to control reset and ISP.
+1.88   2013-04-25 Torsten Lang, Uwe Schneider GmbH
+                  Fixed answer evaluation
+                  Added sector tables for LPC1833
+                  XON/XOFF handling according to LPC manual
+                  Changed COMMTIMEOUTS settings according to MS help
+                  Changed waiting code from tick counting to system time usage
+                  Set MM timers to 1ms resolution (otherwise waiting in the serial driver is limited to multiples of 10ms)
+                  Send all commands with <CR><LF> according to NXP UM
+                  Change answer evaluation to match at least LPC17xx and LPC18xx families (LPC17xx just mirrors the first character of
+                    the LF sequence, LPC18xx mirrors the first character and adds an <LF> which then will lead to a <CR><LF><LF>
+                    sequence, all other lines are terminated with <CR><LF> as documented)
+                  Change answer formatting by filtering leading <LF> characters because they can be left over from a previous response
+                  Store residual data of an answer (required for the J command which will deliver two words in case of the LPC18xx)
+                  Expanded configuration table by second identification word and usage flag
+                  Do a two stage scan in case that a device with two identification words is detected
+1.89   2013-06-27 Martin Maurer
+                  Thanks to Manuel and Henri for bugfixes and speed-ups
+                  Bugfix: In case of LPC8XX, XON/XOFF handling must be switched off,
+                  otherwise download gets broken, because XON/XOFF are filtered out...
+1.90   2013-06-27 Martin Maurer
+                  Add checksum calculation for LPC8XX
+                  winmm function only available in MSVC, put ifdefs around
+                  Workaround for lost characters of LPC8XX
+1.91   2013-06-28 Torsten Lang, Uwe Schneider GmbH
+                  Minor bugfix for the residual data handling
+1.92   2013-06-29 Martin Maurer
+                  Thanks to Phil for reporting Linux compile errors
+                  Update README
+1.93   2013-07-05 Andrew Pines
+                  changed Makefile to exclude "-static" from CFLAGS when building for OS X
+                  changed serial timeout for *nix to use select
+                  added -boothold argument to assert ISP throughtout programming sequence
+                  added support for -try<n> command line argument (was in help, didn't actually work)
+1.94   2013-08-10 Martin Maurer
+                  Add (assumed) part id values for LPC4333 and LPC4337
+                  Correct table entries for all LPC43xx part id numbers
+                  (missing flag for two word part ids and second word for LPC4333 and LPC4353)
+                  Merge in optimization (thanks to Stefan) to skip empty sectors
+                  Correct help output of "-boothold"
+                  Add workaround table entry for LPC4357 and word1 equal to 0x0EF60000
+                  Bugfix for wrong check of word0/word1 combination
+                  Correct a lot entries in lpc property table of LPC18xx and LPC43xx
+                  LPC18xx and LPC43xx: Add warning message, that wipe erases only bank A
 
 */
 
 // Please don't use TABs in the source code !!!
 
 // Don't forget to update the version string that is on the next line
-#define VERSION_STR "1.85"
+#define VERSION_STR "1.94"
 
 #if defined COMPILE_FOR_WINDOWS || defined COMPILE_FOR_CYGWIN
 static char RxTmpBuf[256];        // save received data to this buffer for half-duplex
@@ -369,18 +417,6 @@ static int AddFileHex(ISP_ENVIRONMENT *IspEnvironment, const char *arg);
 static int AddFileBinary(ISP_ENVIRONMENT *IspEnvironment, const char *arg);
 static int LoadFile(ISP_ENVIRONMENT *IspEnvironment, const char *filename, int FileFormat);
 
-#define ERR_RECORD_TYPE_LOADFILE	55 /** File record type not yet implemented. */
-#define ERR_ALLOC_FILE_LIST 60
-#define ERR_FILE_OPEN_HEX	61	/**< Couldn't open hex file. */
-#define ERR_FILE_SIZE_HEX	62	/**< Unexpected hex file size. */
-#define ERR_FILE_ALLOC_HEX	63	/**< Couldn't allocate enough memory for hex file. */
-#define ERR_FILE_ALLOC_BIN	64	/**< Couldn't allocate enough memory for bin file. */
-#define ERR_FILE_RECST_HEX	65	/**< Can't find start of record indicator for Intel Hex file.*/
-#define ERR_FILE_OPEN_BIN	66	/**< Couldn't open binary file. */
-#define ERR_FILE_SIZE_BIN	67	/**< Unexpected binary file size. */
-#define ERR_FILE_WRITE_BIN	68	/**< Couldn't write debug binary file to disk. How's that for ironic? */
-#define ERR_MEMORY_RANGE    69  /**< Out of memory range. */
-
 /************* Portability layer. Serial and console I/O differences    */
 /* are taken care of here.                                              */
 
@@ -390,6 +426,11 @@ static void OpenSerialPort(ISP_ENVIRONMENT *IspEnvironment)
     DCB    dcb;
     COMMTIMEOUTS commtimeouts;
 
+#ifdef _MSC_VER
+    /* Torsten Lang 2013-05-06 Switch to higher timer resolution (we want to use 1ms timeouts in the serial device driver!) */
+    (void)timeBeginPeriod(1UL);
+#endif // _MSC_VER
+
     IspEnvironment->hCom = CreateFile(IspEnvironment->serial_port, GENERIC_READ | GENERIC_WRITE,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
 
     if (IspEnvironment->hCom == INVALID_HANDLE_VALUE)
@@ -406,8 +447,8 @@ static void OpenSerialPort(ISP_ENVIRONMENT *IspEnvironment)
     dcb.StopBits    = ONESTOPBIT;
     dcb.Parity      = NOPARITY;
     dcb.fDtrControl = DTR_CONTROL_DISABLE;
-    dcb.fOutX       = FALSE;
-    dcb.fInX        = FALSE;
+    dcb.fOutX       = TRUE; // TL TODO - according to LPC manual! FALSE;
+    dcb.fInX        = TRUE; // TL TODO - according to LPC manual! FALSE;
     dcb.fNull       = FALSE;
     dcb.fRtsControl = RTS_CONTROL_DISABLE;
 
@@ -440,8 +481,9 @@ static void OpenSerialPort(ISP_ENVIRONMENT *IspEnvironment)
 
     SetCommMask(IspEnvironment->hCom, EV_RXCHAR | EV_TXEMPTY);
 
+    // Torsten Lang 2013-05-06: ReadFile may hang indefinitely with MAXDWORD/0/1/0/0 on FTDI devices!!!
     commtimeouts.ReadIntervalTimeout         = MAXDWORD;
-    commtimeouts.ReadTotalTimeoutMultiplier  =    0;
+    commtimeouts.ReadTotalTimeoutMultiplier  = MAXDWORD;
     commtimeouts.ReadTotalTimeoutConstant    =    1;
     commtimeouts.WriteTotalTimeoutMultiplier =    0;
     commtimeouts.WriteTotalTimeoutConstant   =    0;
@@ -550,6 +592,11 @@ static void OpenSerialPort(ISP_ENVIRONMENT *IspEnvironment)
 static void CloseSerialPort(ISP_ENVIRONMENT *IspEnvironment)
 {
     CloseHandle(IspEnvironment->hCom);
+
+#ifdef _MSC_VER
+    /* Torsten Lang 2013-05-06 Switch back timer resolution */
+    (void)timeEndPeriod(1UL);
+#endif // _MSC_VER
 }
 
 #endif // defined COMPILE_FOR_WINDOWS || defined COMPILE_FOR_CYGWIN
@@ -565,6 +612,60 @@ static void CloseSerialPort(ISP_ENVIRONMENT *IspEnvironment)
 }
 #endif // defined COMPILE_FOR_LINUX
 
+#if defined COMPILE_FOR_WINDOWS || defined COMPILE_FOR_CYGWIN
+void ControlXonXoffSerialPort(ISP_ENVIRONMENT *IspEnvironment, unsigned char XonXoff)
+{
+    DCB dcb;
+
+    GetCommState(IspEnvironment->hCom, &dcb);
+
+    if(XonXoff)
+    {
+        dcb.fOutX = TRUE;
+        dcb.fInX  = TRUE;
+    }
+    else
+    {
+        dcb.fOutX = FALSE;
+        dcb.fInX  = FALSE;
+    }
+
+    if (SetCommState(IspEnvironment->hCom, &dcb) == 0)
+    {
+        DebugPrintf(1, "Can't set XonXoff ! - Error: %ld", GetLastError());
+        exit(3);
+    }
+}
+
+#endif // defined COMPILE_FOR_WINDOWS || defined COMPILE_FOR_CYGWIN
+
+#if defined COMPILE_FOR_LINUX
+void ControlXonXoffSerialPort(ISP_ENVIRONMENT *IspEnvironment, unsigned char XonXoff)
+{
+    if(tcgetattr(IspEnvironment->fdCom, &IspEnvironment->newtio))
+    {
+       DebugPrintf(1, "Could not get serial port behaviour\n");
+       exit(3);
+    }
+
+    if(XonXoff)
+    {
+      IspEnvironment->newtio.c_iflag |= IXON;
+      IspEnvironment->newtio.c_iflag |= IXOFF;
+    }
+    else
+    {
+      IspEnvironment->newtio.c_iflag &= ~IXON;
+      IspEnvironment->newtio.c_iflag &= ~IXOFF;
+    }
+
+    if(tcsetattr(IspEnvironment->fdCom, TCSANOW, &IspEnvironment->newtio))
+    {
+       DebugPrintf(1, "Could not set serial port behaviour\n");
+       exit(3);
+    }
+}
+#endif // defined COMPILE_FOR_LINUX
 
 /***************************** SendComPortBlock *************************/
 /**  Sends a block of bytes out the opened com port.
@@ -638,6 +739,7 @@ void SendComPort(ISP_ENVIRONMENT *IspEnvironment, const char *s)
 /**  Performs a timer tick.  In this simple case all we do is count down
 with protection against underflow and wrapping at the low end.
 */
+#if defined COMPILE_FOR_WINDOWS || defined COMPILE_FOR_CYGWIN
 static void SerialTimeoutTick(ISP_ENVIRONMENT *IspEnvironment)
 {
     if (IspEnvironment->serial_timeout_count <= 1)
@@ -649,7 +751,7 @@ static void SerialTimeoutTick(ISP_ENVIRONMENT *IspEnvironment)
         IspEnvironment->serial_timeout_count--;
     }
 }
-
+#endif
 
 /***************************** ReceiveComPortBlock **********************/
 /**  Receives a buffer from the open com port. Returns all the characters
@@ -695,19 +797,45 @@ static void ReceiveComPortBlock(ISP_ENVIRONMENT *IspEnvironment,
 
 #endif // defined COMPILE_FOR_WINDOWS || defined COMPILE_FOR_CYGWIN
 
-#if defined COMPILE_FOR_LINUX || defined COMPILE_FOR_LPC21
+#if defined COMPILE_FOR_LPC21
 
     *real_size = read(IspEnvironment->fdCom, answer, max_size);
 
+#endif // defined COMPILE_FOR_LPC21
+
+#if defined COMPILE_FOR_LINUX
+    {
+        fd_set
+            readSet;
+        struct timeval
+            timeVal;
+
+        FD_ZERO(&readSet);                             // clear the set
+        FD_SET(IspEnvironment->fdCom,&readSet);        // add this descriptor to the set
+        timeVal.tv_sec=0;                              // set up the timeout waiting for one to come ready (500ms)
+        timeVal.tv_usec=500*1000;
+        if(select(FD_SETSIZE,&readSet,NULL,NULL,&timeVal)==1)    // wait up to 500 ms or until our data is ready
+        {
+            *real_size=read(IspEnvironment->fdCom, answer, max_size);
+        }
+        else
+        {
+            // timed out, show no characters received and timer expired
+            *real_size=0;
+            IspEnvironment->serial_timeout_count=0;
+        }
+    }
 #endif // defined COMPILE_FOR_LINUX
 
     sprintf(tmp_string, "Read(Length=%ld): ", (*real_size));
     DumpString(5, answer, (*real_size), tmp_string);
 
+#if defined COMPILE_FOR_WINDOWS || defined COMPILE_FOR_CYGWIN
     if (*real_size == 0)
     {
         SerialTimeoutTick(IspEnvironment);
     }
+#endif
 }
 
 
@@ -729,8 +857,12 @@ static void SerialTimeoutSet(ISP_ENVIRONMENT *IspEnvironment, unsigned timeout_m
     IspEnvironment->serial_timeout_count = timeout_milliseconds / 100;
 #elif defined COMPILE_FOR_LPC21
     IspEnvironment->serial_timeout_count = timeout_milliseconds * 200;
+#else
+#ifdef _MSC_VER
+    IspEnvironment->serial_timeout_count = timeGetTime() + timeout_milliseconds;
 #else
     IspEnvironment->serial_timeout_count = timeout_milliseconds;
+#endif // _MSC_VER
 #endif
 }
 
@@ -743,10 +875,24 @@ static void SerialTimeoutSet(ISP_ENVIRONMENT *IspEnvironment, unsigned timeout_m
 */
 static int SerialTimeoutCheck(ISP_ENVIRONMENT *IspEnvironment)
 {
+#if defined COMPILE_FOR_WINDOWS || defined COMPILE_FOR_CYGWIN
+#ifdef _MSC_VER
+    if ((signed long)(IspEnvironment->serial_timeout_count - timeGetTime()) < 0)
+    {
+        return 1;
+    }
+#else
     if (IspEnvironment->serial_timeout_count == 0)
     {
         return 1;
     }
+#endif // _MSC_VER
+#else
+    if (IspEnvironment->serial_timeout_count == 0)
+    {
+        return 1;
+    }
+#endif
     return 0;
 }
 
@@ -1017,7 +1163,16 @@ void DebugPrintf(int level, const char *fmt, ...)
 /***************************** ReceiveComPort ***************************/
 /**  Receives a buffer from the open com port. Returns when the buffer is
 filled, the numer of requested linefeeds has been received or the timeout
-period has passed
+period has passed. The bootloaders may send 0x0d,0x0a,0x0a or 0x0d,0x0a as
+linefeed pattern
+2013-06-28 Torsten Lang
+Note: We *could* filter out surplus 0x0a characters like in <CR><LF><LF>
+but as we don't know how the individual bootloader behaves we don't want
+to wait for possible surplus <LF> (which would slow down the transfer).
+Thus, we just terminate after the expected number of <CR><LF> sequences
+and leave it to the command handler in lpcprog.c to filter out surplus
+<LF> characters which then occur as leading character in answers or
+echoed commands.
 \param [in] ISPEnvironment.
 \param [out] Answer buffer to hold the bytes read from the serial port.
 \param [in] MaxSize the size of buffer pointed to by Answer.
@@ -1039,42 +1194,84 @@ void ReceiveComPort(ISP_ENVIRONMENT *IspEnvironment,
     int eof = 0;
     unsigned long p;
     unsigned char *Answer;
+    unsigned char *endPtr;
     char tmp_string[32];
+    static char residual_data[128] = {'\0'};
+    int lf = 0;
 
-    Answer = (unsigned char*) Ans;
+    Answer  = (unsigned char*) Ans;
 
     SerialTimeoutSet(IspEnvironment, timeOutMilliseconds);
 
-    (*RealSize) = 0;
+    *RealSize = 0;
+    endPtr = NULL;
 
     do
     {
-        ReceiveComPortBlock(IspEnvironment, Answer + (*RealSize), MaxSize - 1 - (*RealSize), &tmp_realsize);
+        if (residual_data[0] == '\0')
+        {
+            /* Receive new data */
+            ReceiveComPortBlock(IspEnvironment, Answer + (*RealSize), MaxSize - 1 - (*RealSize), &tmp_realsize);
+        }
+        else
+        {
+            /* Take over any old residual data */
+            strcpy((char *)Answer, residual_data);
+            tmp_realsize = strlen((char *)Answer);
+            residual_data[0] = '\0';
+        }
 
         if (tmp_realsize != 0)
         {
             for (p = (*RealSize); p < (*RealSize) + tmp_realsize; p++)
             {
+                /* Torsten Lang 2013-05-06 Scan for 0x0d,0x0a,0x0a and 0x0d,0x0a as linefeed pattern */
                 if (Answer[p] == 0x0a)
                 {
-                    nr_of_0x0A++;
+                    if (lf != 0)
+                    {
+                        nr_of_0x0A++;
+                        lf = 0;
+                        if (nr_of_0x0A >= WantedNr0x0A)
+                        {
+                            endPtr = &Answer[p+1];
+                        }
+                    }
                 }
                 else if (Answer[p] == 0x0d)
                 {
                     nr_of_0x0D++;
+                    lf = 1;
                 }
                 else if (((signed char) Answer[p]) < 0)
                 {
                     eof = 1;
+                    lf  = 0;
+                }
+                else if (lf != 0)
+                {
+                    nr_of_0x0D++;
+                    nr_of_0x0A++;
+                    lf = 0;
+                    if (nr_of_0x0A >= WantedNr0x0A)
+                    {
+                        endPtr = &Answer[p+1];
+                    }
                 }
             }
+            (*RealSize) += tmp_realsize;
         }
+    } while (((*RealSize) < MaxSize) && (SerialTimeoutCheck(IspEnvironment) == 0) && (nr_of_0x0A < WantedNr0x0A) && !eof);
 
-        (*RealSize) += tmp_realsize;
-
-    } while (((*RealSize) < MaxSize) && (SerialTimeoutCheck(IspEnvironment) == 0) && (nr_of_0x0A < WantedNr0x0A) && (nr_of_0x0D < WantedNr0x0A) && !eof);
-
-    Answer[(*RealSize)] = 0;
+    /* Torsten Lang 2013-05-06 Store residual data and cut answer after expected nr. of 0x0a */
+    Answer[*RealSize] = '\0';
+    if (endPtr != NULL)
+    {
+        strcpy(residual_data, (char *)endPtr);
+        *endPtr = '\0';
+        /* Torsten Lang 2013-06-28 Update size info */
+        *RealSize = endPtr-Answer;
+    }
 
     sprintf(tmp_string, "Answer(Length=%ld): ", (*RealSize));
     DumpString(3, Answer, (*RealSize), tmp_string);
@@ -1184,13 +1381,38 @@ static void ReadArguments(ISP_ENVIRONMENT *IspEnvironment, unsigned int argc, ch
                 continue;
             }
 
-            if (stricmp(argv[i], "-DoNotStart") == 0)
+            if (stricmp(argv[i], "-boothold") == 0)
+            {
+                IspEnvironment->BootHold = 1;
+                DebugPrintf(3, "hold EnableBootLoader asserted throughout programming sequence.\n");
+                continue;
+            }
+
+            if (stricmp(argv[i], "-donotstart") == 0)
             {
                 IspEnvironment->DoNotStart = 1;
                 DebugPrintf(3, "Do NOT start MCU after programming.\n");
                 continue;
             }
 
+            if(strnicmp(argv[i],"-try", 4) == 0)
+            {
+                int
+                    retry;
+                retry=atoi(&argv[i][4]);
+                if(retry>0)
+                {
+                    IspEnvironment->nQuestionMarks=retry;
+                    DebugPrintf(3, "Retry count: %d.\n", IspEnvironment->nQuestionMarks);
+                }
+                else
+                {
+                    fprintf(stderr,"invalid argument for -try: \"%s\"\n",argv[i]);
+                }
+                continue;
+            }
+
+
             if (stricmp(argv[i], "-control") == 0)
             {
                 IspEnvironment->ControlLines = 1;
@@ -1297,9 +1519,9 @@ static void ReadArguments(ISP_ENVIRONMENT *IspEnvironment, unsigned int argc, ch
     {
         DebugPrintf(2, "\n"
                        "Portable command line ISP\n"
-                       "for NXP LPC1000 / LPC2000 family and Analog Devices ADUC 70xx\n"
+                       "for NXP LPC family and Analog Devices ADUC 70xx\n"
                        "Version " VERSION_STR " compiled for " COMPILED_FOR ": " __DATE__ ", " __TIME__ "\n"
-                       "Copyright (c) by Martin Maurer, 2003-2011, Email: Martin.Maurer@clibb.de\n"
+                       "Copyright (c) by Martin Maurer, 2003-2013, Email: Martin.Maurer@clibb.de\n"
                        "Portions Copyright (c) by Aeolus Development 2004, www.aeolusdevelopment.com\n"
                        "\n");
 
@@ -1319,6 +1541,7 @@ static void ReadArguments(ISP_ENVIRONMENT *IspEnvironment, unsigned int argc, ch
                        "         -wipe        Erase entire device before upload\n"
                        "         -control     for controlling RS232 lines for easier booting\n"
                        "                      (Reset = DTR, EnableBootLoader = RTS)\n"
+                       "         -boothold    hold EnableBootLoader asserted throughout sequence\n"
 #ifdef INTEGRATED_IN_WIN_APP
                        "         -nosync      Do not synchronize device via '?'\n"
 #endif
@@ -1332,7 +1555,7 @@ static void ReadArguments(ISP_ENVIRONMENT *IspEnvironment, unsigned int argc, ch
                        "         -halfduplex  use halfduplex serial communication (i.e. with K-Line)\n"
                        "         -ADARM       for downloading to an Analog Devices\n"
                        "                      ARM microcontroller ADUC70xx\n"
-                       "         -NXPARM      for downloading to a NXP LPC1xxx/LPC2xxx (default)\n");
+                       "         -NXPARM      for downloading to a chip of NXP LPC family (default)\n");
 
         exit(1);
     }
@@ -1355,6 +1578,87 @@ run mode.
 */
 void ResetTarget(ISP_ENVIRONMENT *IspEnvironment, TARGET_MODE mode)
 {
+#if defined(__linux__) && defined(GPIO_RST) && defined(GPIO_ISP)
+
+// This code section allows using Linux GPIO pins to control the -RST and -ISP
+// signals of the target microcontroller.
+//
+// Build lpc21isp to use Linux GPIO like this:
+//
+// make CFLAGS="-Wall -DGPIO_ISP=23 -DGPIO_RST=18"
+//
+// The GPIO pins must be pre-configured in /etc/rc.local (or other startup script)
+// similar to the following:
+//
+// # Configure -ISP signal
+// echo 23 >/sys/class/gpio/export
+// echo out >/sys/class/gpio/gpio23/direction
+// echo 1 >/sys/class/gpio/gpio23/value
+// chown root.gpio /sys/class/gpio/gpio23/value
+// chmod 660 /sys/class/gpio/gpio23/value
+//
+// # Configure -RST signal
+// echo 18 >/sys/class/gpio/export
+// echo out >/sys/class/gpio/gpio18/direction
+// echo 1 >/sys/class/gpio/gpio18/value
+// chown root.gpio /sys/class/gpio/gpio18/value
+// chmod 660 /sys/class/gpio/gpio18/value
+//
+// Then if the user is a member of the gpio group, lpc21isp will not requre any
+// special permissions to access the GPIO signals.
+
+  char gpio_isp_filename[256];
+  char gpio_rst_filename[256];
+  int gpio_isp;
+  int gpio_rst;
+
+  memset(gpio_isp_filename, 0, sizeof(gpio_isp_filename));
+  sprintf(gpio_isp_filename, "/sys/class/gpio/gpio%d/value", GPIO_ISP);
+
+  memset(gpio_rst_filename, 0, sizeof(gpio_rst_filename));
+  sprintf(gpio_rst_filename, "/sys/class/gpio/gpio%d/value", GPIO_RST);
+
+  gpio_isp = open(gpio_isp_filename, O_WRONLY);
+  if (gpio_isp < 0)
+  {
+    fprintf(stderr, "ERROR: open() for %s failed, %s\n", gpio_isp_filename, strerror(errno));
+    exit(1);
+  }
+
+  gpio_rst = open(gpio_rst_filename, O_WRONLY);
+  if (gpio_rst < 0)
+  {
+    fprintf(stderr, "ERROR: open() for %s failed, %s\n", gpio_rst_filename, strerror(errno));
+    exit(1);
+  }
+
+  switch (mode)
+  {
+    case PROGRAM_MODE :
+      write(gpio_isp, "0\n", 2);  // Assert -ISP
+      Sleep(100);
+      write(gpio_rst, "0\n", 2);  // Assert -RST
+      Sleep(500);
+      write(gpio_rst, "1\n", 2);  // Deassert -RST
+      Sleep(100);
+      write(gpio_isp, "1\n", 2);  // Deassert -ISP
+      Sleep(100);
+      break;;
+
+    case RUN_MODE :
+      write(gpio_rst, "0\n", 2);  // Assert -RST
+      Sleep(500);
+      write(gpio_rst, "1\n", 2);  // Deassert -ISP
+      Sleep(100);
+      break;;
+  }
+
+  close(gpio_isp);
+  close(gpio_rst);
+
+  return;
+#endif
+
     if (IspEnvironment->ControlLines)
     {
         switch (mode)
@@ -1370,7 +1674,10 @@ void ResetTarget(ISP_ENVIRONMENT *IspEnvironment, TARGET_MODE mode)
             Sleep(500);
             // Clear the RTS line after having reset the micro
             // Needed for the "GO <Address> <Mode>" ISP command to work */
-            ControlModemLines(IspEnvironment, 0, 0);
+            if(!IspEnvironment->BootHold)
+            {
+                ControlModemLines(IspEnvironment, 0, 0);
+            }
             break;
 
         /* Reset and start uploaded program                     */
@@ -1685,8 +1992,8 @@ void ReadHexFile(ISP_ENVIRONMENT *IspEnvironment)
 /***************************** LoadFile *********************************/
 /**  Loads the requested file to download into memory.
 \param [in] IspEnvironment  structure containing input filename
-\param [in] filename	the name of the file to read in.
-\param [in] FileFormat	the format of the file to read in (FORMAT_HEX or FORMAT_BINARY)
+\param [in] filename  the name of the file to read in.
+\param [in] FileFormat  the format of the file to read in (FORMAT_HEX or FORMAT_BINARY)
 \return 0 if successful, otherwise an error code.
 */
 static int LoadFile(ISP_ENVIRONMENT *IspEnvironment, const char *filename, int FileFormat)
@@ -1944,7 +2251,7 @@ static int LoadFile(ISP_ENVIRONMENT *IspEnvironment, const char *filename, int F
             close(fdout);
         }
 
-        free( FileContent);		// Done with file contents
+        free( FileContent);   // Done with file contents
     }
     else // FORMAT_BINARY
     {
@@ -1972,10 +2279,10 @@ static int LoadFiles1(ISP_ENVIRONMENT *IspEnvironment, const FILE_LIST *file)
         DebugPrintf( 3, "Follow file list %s\n", file->name);
 
         ret_val = LoadFiles1( IspEnvironment, file->prev);
-		if( ret_val != 0)
-		{
-			return ret_val;
-		}
+    if( ret_val != 0)
+    {
+      return ret_val;
+    }
     }
 
     DebugPrintf( 3, "Attempt to read File %s\n", file->name);
@@ -1985,11 +2292,11 @@ static int LoadFiles1(ISP_ENVIRONMENT *IspEnvironment, const FILE_LIST *file)
     }
     else
     {
-		ret_val = LoadFile(IspEnvironment, file->name, FORMAT_BINARY);
+    ret_val = LoadFile(IspEnvironment, file->name, FORMAT_BINARY);
     }
     if( ret_val != 0)
     {
-		return ret_val;
+    return ret_val;
     }
 
     return 0;
@@ -2003,15 +2310,15 @@ static int LoadFiles1(ISP_ENVIRONMENT *IspEnvironment, const FILE_LIST *file)
 */
 static int LoadFiles(ISP_ENVIRONMENT *IspEnvironment)
 {
-	int ret_val;
+  int ret_val;
 
     ret_val = LoadFiles1(IspEnvironment, IspEnvironment->f_list);
     if( ret_val != 0)
     {
-		exit(1); // return ret_val;
+    exit(1); // return ret_val;
     }
 
-	DebugPrintf( 2, "Image size : %ld\n", IspEnvironment->BinaryLength);
+  DebugPrintf( 2, "Image size : %ld\n", IspEnvironment->BinaryLength);
 
     // check length to flash for correct alignment, can happen with broken ld-scripts
     if (IspEnvironment->BinaryLength % 4 != 0)
@@ -2023,11 +2330,11 @@ static int LoadFiles(ISP_ENVIRONMENT *IspEnvironment)
         IspEnvironment->BinaryLength = NewBinaryLength;
     }
 
-	// When debugging is switched on, output result of conversion to file debugout.bin
+  // When debugging is switched on, output result of conversion to file debugout.bin
     if(debug_level >= 4)
     {
          int fdout;
-		 DebugPrintf( 1, "Dumping image file.\n");
+     DebugPrintf( 1, "Dumping image file.\n");
          fdout = open("debugout.bin", O_RDWR | O_BINARY | O_CREAT | O_TRUNC, 0777);
          write(fdout, IspEnvironment->BinaryContent, IspEnvironment->BinaryLength);
          close(fdout);
@@ -2129,6 +2436,7 @@ int main(int argc, char *argv[])
     IspEnvironment.ProgramChip = TRUE;                        // Default to Programming the chip
     IspEnvironment.nQuestionMarks = 100;
     IspEnvironment.DoNotStart = 0;
+    IspEnvironment.BootHold = 0;
     ReadArguments(&IspEnvironment, argc, argv);               // Read and parse the command line
 
     return PerformActions(&IspEnvironment);                   // Do as requested !
@@ -2169,27 +2477,3 @@ void DumpString(int level, const void *b, size_t size, const char *prefix_string
     DebugPrintf(level, "'\n");
 }
 
-#if !defined COMPILE_FOR_LPC21
-int lpctest(char* FileName)
-{
-    ISP_ENVIRONMENT IspEnvironment;
-
-    // Initialize debug level
-    debug_level = 2;
-
-    // Initialize ISP Environment
-    memset(&IspEnvironment, 0, sizeof(IspEnvironment));        // Clear the IspEnviroment to a known value
-    IspEnvironment.micro        = NXP_ARM;                     // Default Micro
-    IspEnvironment.FileFormat   = FORMAT_HEX;                  // Default File Format
-    IspEnvironment.ProgramChip  = TRUE;                        // Default to Programming the chip
-    // IspEnvironment.input_file   = FileName;
-    IspEnvironment.ControlLines = TRUE;
-    IspEnvironment.serial_port  = "COM2";
-    IspEnvironment.baud_rate    = "19200";
-    IspEnvironment.nQuestionMarks = 100;
-    IspEnvironment.DoNotStart = 0;
-    strcpy(IspEnvironment.StringOscillator, "25000");
-
-    return PerformActions(&IspEnvironment);                    // Do as requested !
-}
-#endif
